@@ -1,6 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 1998-2000, 2008, 2010, 2012, 2016-2018 Sergey
-   Poznyakoff
+   Copyright (C) 1998-2021 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,7 +24,7 @@ int single_process; /* Single process mode */
 char *config_file = SYSCONFIG "/dicod.conf" ;
 int config_lint_option; /* Check configuration file syntax and exit. */
 /* Location of the pidfile */
-char *pidfile_name = LOCALSTATEDIR "/run/dicod.pid";
+char *pidfile_name = DEFAULT_PIDFILE_NAME;
 
 /* Operation mode */
 int mode = MODE_DAEMON;
