@@ -1,6 +1,6 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
-# Copyright (C) 2008-2018 Sergey Poznyakoff
+# Copyright (C) 2008-2021 Sergey Poznyakoff
 # Copyright (C) 2008, 2012 Wojciech Polak
 # 
 # GNU Dico is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
 # 
 # You should have received a copy of the GNU General Public License
 # along with GNU Dico.  If not, see <http://www.gnu.org/licenses/>.
-
+from __future__ import print_function
 import dico
 import sys
 
@@ -41,13 +41,13 @@ class DicoResult:
         
 class DicoDefineResult (DicoResult):
     def output (self, n):
-        print "%d. %s" % (n + 1, self.result[n])
-        print "---------",
+        print("%d. %s" % (n + 1, self.result[n]))
+        print("---------", end="")
 
 class DicoMatchResult (DicoResult):
     def output (self, n):
         sys.stdout.softspace = 0
-        print self.result[n],
+        print(self.result[n], end="")
 
 class DicoModule:
     adict =  {}
@@ -55,6 +55,7 @@ class DicoModule:
     filename = ''
     mod_descr = ''
     mod_info = []
+    mime_headers = []
     langlist = ()
 
     def __init__ (self, *argv):
@@ -63,7 +64,10 @@ class DicoModule:
     
     def open (self, dbname):
         self.dbname = dbname
-        file = open (self.filename, "r")
+        if sys.version_info[0] <= 2:
+            file = open (self.filename, "r")
+        else:
+            file = open (self.filename, "r", encoding="utf8")
         for line in file:
             if line.startswith ('--'):
                 continue
@@ -80,6 +84,9 @@ class DicoModule:
                 else:
                     self.langlist = (s[0].split (), s[1].split ())
                 continue
+            if line.startswith ('mime: '):
+                self.mime_headers.append(line[6:].strip (' \n'))
+                continue
             f = line.strip (' \n').split (' ', 1)
             if len (f) == 2:
                 self.adict[f[0].lower()] = f[1].strip (' ')
@@ -99,13 +106,13 @@ class DicoModule:
         return '\n'.join (self.mod_info)
     
     def define_word (self, word):
-        if self.adict.has_key (word):
+        if word in self.adict:
             return DicoDefineResult ([self.adict[word]])
         return False
     
     def match_word (self, strat, key):
         if strat.name == "exact":
-            if self.adict.has_key (key.word.lower ()):
+            if key.word.lower () in self.adict:
                 return DicoMatchResult ([self.adict[key.word.lower()]])
         elif strat.name == "prefix":
             res = []
@@ -119,7 +126,7 @@ class DicoModule:
             for k in self.adict:
                 if strat.select (k, key):
                     res.append (k)
-            if res.count > 0:
+            if res.count() > 0:
                 return res
         return False
 
@@ -132,4 +139,6 @@ class DicoModule:
 
     def compare_count (self, rh):
         return rh.compcount
-    
+
+    def db_mime_header (self):
+        return '\n'.join(self.mime_headers)
