@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2008-2018 Sergey Poznyakoff
+   Copyright (C) 2008-2021 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,9 +14,7 @@
    You should have received a copy of the GNU General Public License
    along with GNU Dico.  If not, see <http://www.gnu.org/licenses/>. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 #include <dico.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -214,7 +212,6 @@ _guile_dico_key_free(SCM message_smob)
 {
     struct dico_key *kp = (struct dico_key *) SCM_CDR (message_smob);
     dico_key_deinit(kp);
-    free(kp);
     return 0;
 }
 
@@ -286,9 +283,8 @@ _make_strategy(const dico_strategy_t strat)
 static size_t
 _guile_strategy_free(SCM message_smob)
 {
-    struct _guile_strategy *sp =
-	(struct _guile_strategy *) SCM_CDR (message_smob);
-    free(sp);
+    /* Nothing to free in struct _guile_strategy: the strat member is
+       constant */
     return 0;
 }
 
@@ -815,7 +811,7 @@ mod_init(int argc, char **argv)
 }
 
 static dico_handle_t
-mod_init_db(const char *dbname, int argc, char **argv, void *extra)
+mod_init_db(const char *dbname, int argc, char **argv)
 {
     struct _guile_database *db;
     int i;
